com.biffweb.fx

Remove effects from your application logic with state machines.

SCHEMA

:biff.fx/handlers {:com.example/do-something (fn [ctx & args]), ...}

A map from effect keywords to handler functions.

:biff.fx/get-handlers (fn []) => {...}

A function that returns a :biff.fx/handlers map. Intended for use by :biff.core/init functions.

:biff.fx/next keyword

A key in the state->fn map passed to biff.fx/machine.

:biff.fx/now instant

The current time.

:biff.fx/seed long

A random seed. Use it like (java.util.Random. seed)

Remove effects from your application logic with state machines.

SCHEMA

:biff.fx/handlers
{:com.example/do-something (fn [ctx & args]),
 ...}

  A map from effect keywords to handler functions.


:biff.fx/get-handlers
(fn []) => {...}

  A function that returns a :biff.fx/handlers map. Intended for use by
  :biff.core/init functions.


:biff.fx/next
keyword

  A key in the state->fn map passed to biff.fx/machine.


:biff.fx/now
instant

  The current time.


:biff.fx/seed
long

  A random seed. Use it like (java.util.Random. seed)
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries